'''
@author: Sebastian Sudholt
@organization: TU Dortmund University, Dortmund, Germany
@contact: sebastian.sudholt@tu-dortmund.de
'''

import time
import numpy as np
from scipy.cluster.vq import kmeans2, vq

def create_visual_vocabulary(word_list, codebook_size, descriptors_for_clustering):
    print('Extracting Descriptors for Clustering... '),
    cluster_mat = np.zeros((descriptors_for_clustering, 128))
    word_indices = np.random.randint(len(word_list), size=descriptors_for_clustering)
    for cluster_mat_position, word_idx in enumerate(word_indices):
        random_descriptor_idx = np.random.randint(word_list[word_idx][2].shape[0], size=1)[0]
        cluster_mat[cluster_mat_position] = word_list[word_idx][2][random_descriptor_idx]
    print('[ Done ]\nClustering Descriptors (this might take quite a while)... ')
    # run clustering
    start_time = time.time()
    codebook, _ = kmeans2(data=cluster_mat, k=codebook_size, iter=25)
    run_time = time.time() - start_time
    print('[ Done ] (took %f mins.)' % (run_time/60.0))
    return codebook

def quantize_descriptors(word_list, codebook):
    print('Quantizing Descriptors:')
    start_time = time.time()
    quantized_descriptors_list = []
    for idx, word in enumerate(word_list):
        if (idx+1) % 500 == 0 or (idx+1) == len(word_list):
            print('\t[ %4d / %4d ]' % (idx+1, len(word_list)))
        quantized_descriptors_list.append(vq(obs=word[2], code_book=codebook)[0])
    run_time = time.time() - start_time
    print('[ Done ] (took %f mins.)' % (run_time/60.0))
    return quantized_descriptors_list